package com.zcxk.xxljob.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.PageUtil;
import com.alibaba.fastjson.JSON;
import com.zcxk.core.common.enums.StatusEnum;
import com.zcxk.core.common.util.SnowflakeIdWorker;
import com.zcxk.core.utils.DateUtil;
import com.zcxk.rmcp.api.dto.meterreadrecord.MeterReadRecordDetailDto;
import com.zcxk.rmcp.api.dto.readmeter.WaterMeterDto;
import com.zcxk.rmcp.api.enums.ReadStatusEnum;
import com.zcxk.rmcp.core.dao.DeviceMapper;
import com.zcxk.rmcp.core.dao.MeterReadRecordMapper;
import com.zcxk.rmcp.core.mongo.MeterReadRecord;
import com.zcxk.xxljob.common.Constants;
import com.zcxk.xxljob.service.MeterReadRecordService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.*;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author linqingwei
 * @date 2021-08-11 11:36
 */
@Slf4j
@Service
public class MeterReadRecordServiceImpl implements MeterReadRecordService {

    @Resource
    private DeviceMapper deviceMapper;
    @Autowired
    private MongoTemplate mongoTemplate;
    @Autowired
    private SnowflakeIdWorker idWorker;
    @Resource
    private MeterReadRecordMapper meterReadRecordMapper;

    @Override
    public void executeCreateMeterUnReadRecord(Map<String, Object> map, int index, int total) {
        log.info("begin executeCreateMeterUnReadRecord map = {} {} {}",JSON.toJSONString(map),index,total);
        int readDay = MapUtil.getInt(map, "readDay");
        //计算分页
        int deviceTotal = deviceMapper.countWaterMeter();
        int pageSize = PageUtil.totalPage(deviceTotal, total);
        int startIndex = index * pageSize;

        List<WaterMeterDto> waterMeterList = deviceMapper.findWaterMeterListWithPage(startIndex, pageSize);
        batchCreateMeterUnReadRecord(waterMeterList,readDay);
        log.info("end executeCreateMeterUnReadRecord");
    }

    private void batchCreateMeterUnReadRecord(List<WaterMeterDto> waterMeterList, int readDay) {
        if(waterMeterList.isEmpty())return;
        for (WaterMeterDto waterMeter : waterMeterList) {
            saveReadingData(waterMeter,readDay);
        }
    }

    private void saveReadingData(WaterMeterDto waterMeter,int readDay) {
        log.info("begin saveReadingData ,waterMeter = {}" , JSON.toJSONString(waterMeter));
        MeterReadRecord.MeterReadInfo meterReadInfo =  buildMeterReadInfo(readDay);


        //推送数据
        Query pushQuery = new Query();
        pushQuery.addCriteria(Criteria.where("deviceId").is(waterMeter.getId()));
        Update push = new Update();
        push.push("data",meterReadInfo);
        long pushResult = mongoTemplate.updateMulti(pushQuery,push, Constants.METER_READ_RECORD_TABLE).getModifiedCount();
        if(pushResult == 0){
            //推送失败插入数据
            MeterReadRecord meterReadRecord = buildMeterReadRecord(waterMeter,meterReadInfo);
            mongoTemplate.insert(meterReadRecord,Constants.METER_READ_RECORD_TABLE);
        }
        log.info("end saveReadingData");
    }

    private MeterReadRecord.MeterReadInfo buildMeterReadInfo(int readDay) {


        MeterReadRecord.MeterReadInfo readInfo = new MeterReadRecord.MeterReadInfo();
        readInfo.setReadDate(readDay);
        //readInfo.setReadTime(meterData.getReceiveDate());
        readInfo.setReadStatus(ReadStatusEnum.UN_READ.getCode());
        //readInfo.setValveStatus(meterData.getValveStatus());
        //readInfo.setReadData(meterData.getReadData());
        //readInfo.setLastValid(lastMeterData.getReadData());
        readInfo.setLastCost(0d);//用水量
        return readInfo;
    }

    private MeterReadRecord buildMeterReadRecord(WaterMeterDto waterMeter,MeterReadRecord.MeterReadInfo meterReadInfo) {

        List<MeterReadRecord.MeterReadInfo> list = new ArrayList<>();
        list.add(meterReadInfo);

        MeterReadRecord readRecord = new MeterReadRecord();
        readRecord.setId(idWorker.nextId());
        readRecord.setYear(DateUtil.getYear(new Date()));
        readRecord.setTenantId(waterMeter.getTenantId());
        readRecord.setCategoryId(waterMeter.getCategoryId());
        readRecord.setCompanyOrgId(waterMeter.getCompanyOrgId());
        readRecord.setCompanyOrgName(waterMeter.getCompanyName());
        readRecord.setDeptOrgId(waterMeter.getDeptOrgId());
        readRecord.setDeptOrgName(waterMeter.getDeptName());
        readRecord.setCommunityId(waterMeter.getCommunityId());
        readRecord.setCommunityName(waterMeter.getCommunityName());
        //readRecord.setConcentratorId(0);
        //readRecord.setConcentratorCode("");
        //readRecord.setCollectorId(0);
        //readRecord.setCollectorCode("");
        //readRecord.setChannelNumber("");
        readRecord.setDeviceId(waterMeter.getId());
        readRecord.setDeviceNo(waterMeter.getDeviceNo());
        readRecord.setLocation(waterMeter.getDeviceNo());
        readRecord.setProductId(waterMeter.getProductId());
        readRecord.setDeviceModel(waterMeter.getManufacturerName()+"/"+waterMeter.getProductName()+"/"+waterMeter.getProductModel());
        readRecord.setMeterNo(waterMeter.getMeterNo());
        readRecord.setFileNo(waterMeter.getFileNo());
        readRecord.setCreateTime(new Date());
        readRecord.setCreatorName(Constants.SYS_FLAG);
        readRecord.setUpdateTime(new Date());
        readRecord.setData(list);

        return readRecord;
    }


    @Override
    public void executeSyncMeterReadRecord(Map<String, Object> map) {
        log.info("begin executeSyncMeterReadRecord map = {}",JSON.toJSONString(map));
        int readDay = MapUtil.getInt(map, "readDay");
        List<MeterReadRecordDetailDto> list = findMeterReadRecord(readDay);
        List<com.zcxk.rmcp.core.entity.MeterReadRecord> readRecordList = new ArrayList<>();
        if(!list.isEmpty()){
            for (MeterReadRecordDetailDto detail : list) {
                com.zcxk.rmcp.core.entity.MeterReadRecord readRecord = new com.zcxk.rmcp.core.entity.MeterReadRecord();
                readRecord.setId(detail.getId());
                readRecord.setReadDate(detail.getData().getReadDate());
                readRecord.setTenantId(detail.getTenantId());
                readRecord.setCategoryId(detail.getCategoryId());
                readRecord.setProductId(detail.getProductId());
                readRecord.setDeviceId(detail.getDeviceId());
                readRecord.setCommunityId(detail.getCommunityId());
                readRecord.setCompanyOrgId(detail.getCompanyOrgId());
                readRecord.setDeptOrgId(detail.getDeptOrgId());
                readRecord.setConcentratorId(detail.getConcentratorId());
                readRecord.setCollectorId(detail.getCollectorId());
                if(detail.getData().getReadTime()!=null)readRecord.setReadTime(LocalDateTimeUtil.of(detail.getData().getReadTime()));
                readRecord.setReadStatus(detail.getData().getReadStatus());
                if(detail.getData().getReadData()!=null)readRecord.setReadData(detail.getData().getReadData().toString());
                if(detail.getData().getLastValid()!=null)readRecord.setLastValid(detail.getData().getLastValid().toString());
                if(detail.getData().getLastCost()!=null)readRecord.setLastCost(BigDecimal.valueOf(detail.getData().getLastCost()));
                readRecord.setStatus(StatusEnum.OK.getCode());
                readRecord.setCreateDate(LocalDateTime.now());
                readRecord.setUpdateDate(LocalDateTime.now());
                readRecord.setCreateBy(Constants.SYS_FLAG);
                readRecord.setUpdateBy(Constants.SYS_FLAG);
                readRecordList.add(readRecord);
            }
            meterReadRecordMapper.batchInsert(readRecordList);
        }
        log.info("end executeSyncMeterReadRecord");
    }


    private List<MeterReadRecordDetailDto> findMeterReadRecord(Integer readDate){
        List<AggregationOperation> commonOperations = new ArrayList<>();
        // 2.拆开
        UnwindOperation unwind = Aggregation.unwind("data");
        commonOperations.add(unwind);
        // 3.子文档查询
        MatchOperation match2 = Aggregation.match(
                Criteria.where("data.readDate").is(readDate));
        commonOperations.add(match2);

        AggregationOptions aggregationOptions = AggregationOptions.builder().allowDiskUse(true).build();

        AggregationResults<MeterReadRecordDetailDto> durationData =
                mongoTemplate.aggregate(Aggregation.newAggregation(commonOperations).withOptions(aggregationOptions),
                        MeterReadRecord.class, MeterReadRecordDetailDto.class);

        return durationData.getMappedResults();
    }

    @Override
    public void todayReadRateJobHandler(Map<String, Object> map) {
        log.info("begin yesterdayReadRateJobHandler map = {}",JSON.toJSONString(map));
        Date readDay = cn.hutool.core.date.DateUtil.parse(MapUtil.getStr(map, "readDay"),"yyyyMMdd");
        meterReadRecordMapper.todayReadRateJobHandler(readDay);
    }

    @Override
    public void yesterdayReadRateJobHandler(Map<String, Object> map) {
        log.info("begin yesterdayReadRateJobHandler map = {}",JSON.toJSONString(map));
        Date readDay = cn.hutool.core.date.DateUtil.parse(MapUtil.getStr(map, "readDay"),"yyyyMMdd");
        meterReadRecordMapper.yesterdayReadRate(readDay);
    }

    @Override
    public void recent7daysReadRateJobHandler(Map<String, Object> map) {
        log.info("begin recent7daysReadRateJobHandler map = {}",JSON.toJSONString(map));
        Date readDay = cn.hutool.core.date.DateUtil.parse(MapUtil.getStr(map, "readDay"),"yyyyMMdd");
        meterReadRecordMapper.recent7daysReadRate(readDay);
    }

    @Override
    public void lastMonthReadRateJobHandler(Map<String, Object> map) {
        log.info("begin lastMonthReadRateJobHandler map = {}",JSON.toJSONString(map));
        Date readDay = cn.hutool.core.date.DateUtil.parse(MapUtil.getStr(map, "readDay"),"yyyyMMdd");
        meterReadRecordMapper.lastMonthReadRate(readDay);
    }
}
